@@ -18,10 +18,12 @@
  ***************************************************************************/
 
 #include "handlerhelper.h"
+#include "imapstreamparser.h"
 #include "storage/datastore.h"
 #include "storage/selectquerybuilder.h"
-
-#include "../libs/imapparser_p.h"
+#include "libs/imapparser_p.h"
+#include "libs/protocol_p.h"
+#include "handler.h"
 
 using namespace Akonadi;
 
@@ -54,7 +56,7 @@ Collection HandlerHelper::collectionFromIdOrName(const QByteArray & id)
     if ( col.isValid() )
       qb.addValueCondition( Collection::parentIdColumn(), Query::Equals, col.id() );
     else
-      qb.addValueCondition( Collection::parentIdColumn(), Query::Equals, 0 );
+      qb.addValueCondition( Collection::parentIdColumn(), Query::Is, QVariant() );
     if ( !qb.exec() )
       return Collection();
     Collection::List list = qb.result();
@@ -184,7 +186,8 @@ QByteArray HandlerHelper::cachePolicyToByteArray(const Collection & col)
   return rv;
 }
 
-QByteArray HandlerHelper::collectionToByteArray( const Collection & col, bool hidden, bool includeStatistics )
+QByteArray HandlerHelper::collectionToByteArray( const Collection & col, bool hidden, bool includeStatistics,
+                                                 int ancestorDepth, const QStack<Collection> &ancestors )
 {
   QByteArray b = QByteArray::number( col.id() ) + ' '
                + QByteArray::number( col.parentId() ) + " (";
@@ -195,8 +198,8 @@ QByteArray HandlerHelper::collectionToByteArray( const Collection & col, bool hi
     b+= "MIMETYPE () ";
   else
     b += "MIMETYPE (" + MimeType::joinByName( col.mimeTypes(), QLatin1String( " " ) ).toLatin1() + ") ";
-  b += "REMOTEID \"" + col.remoteId().toUtf8() + "\" ";
-  b += "RESOURCE \"" + col.resource().name().toUtf8() + "\" ";
+  b += "REMOTEID " + ImapParser::quote( col.remoteId().toUtf8() );
+  b += " RESOURCE " + ImapParser::quote( col.resource().name().toUtf8() ) + ' ';
 
   if ( includeStatistics ) {
       b += "MESSAGES " + QByteArray::number( HandlerHelper::itemCount( col ) ) + ' ';
@@ -205,11 +208,69 @@ QByteArray HandlerHelper::collectionToByteArray( const Collection & col, bool hi
   }
 
   b += HandlerHelper::cachePolicyToByteArray( col ) + ' ';
+  if ( ancestorDepth > 0 )
+    b += HandlerHelper::ancestorsToByteArray( ancestorDepth, ancestors ) + ' ';
 
-  CollectionAttribute::List attrs = col.attributes();
-  foreach ( const CollectionAttribute &attr, attrs )
+  const CollectionAttribute::List attrs = col.attributes();
+  for ( int i = 0; i < attrs.size(); ++i ) {
+    const CollectionAttribute &attr = attrs[i];
     b += attr.type() + ' ' + ImapParser::quote( attr.value() );
+    if ( i != attrs.size() - 1 )
+      b += ' ';
+  }
   b+= ')';
 
   return b;
 }
+
+QByteArray HandlerHelper::ancestorsToByteArray( int ancestorDepth, const QStack<Collection> &_ancestors )
+{
+  QByteArray b;
+  if ( ancestorDepth > 0 ) {
+    b += AKONADI_PARAM_ANCESTORS " (";
+    QStack<Collection> ancestors( _ancestors );
+    for ( int i = 0; i < ancestorDepth; ++i ) {
+      if ( ancestors.isEmpty() ) {
+        b += "(0 \"\")";
+        break;
+      }
+      b += '(';
+      const Collection c = ancestors.pop();
+      b += QByteArray::number( c.id() ) + " \"";
+      b += c.remoteId().toUtf8();
+      b += "\")";
+      if ( i != ancestorDepth - 1 )
+        b += ' ';
+    }
+    b += ')';
+  }
+  return b;
+}
+
+int HandlerHelper::parseDepth( const QByteArray &depth )
+{
+  if ( depth.isEmpty() )
+    throw ImapParserException( "No depth specified" );
+  if ( depth == "INF" )
+    return INT_MAX;
+  bool ok = false;
+  int result = depth.toInt( &ok );
+  if ( !ok )
+    throw ImapParserException( "Invalid depth argument" );
+  return result;
+}
+
+Akonadi::Flag::List Akonadi::HandlerHelper::resolveFlags(const QList< QByteArray >& flagNames)
+{
+  Flag::List flagList;
+  foreach ( const QByteArray &flagName, flagNames ) {
+    Flag flag = Flag::retrieveByName( QString::fromUtf8( flagName ) );
+    if ( !flag.isValid() ) {
+      flag = Flag( QString::fromUtf8( flagName ) );
+      if ( !flag.insert() )
+        throw HandlerException( "Unable to create flag" );
+    }
+    flagList.append( flag );
+  }
+  return flagList;
+}
